!	----------------------------------------------------------------------
!	Module name: InitializationModule.f90
!	----------------------------------------------------------------------


module InitializationModule

	use Globals
	use ValueModule
	use OptimizationMod
	use TauchenMod
	
	
	implicit none
	
	
contains


subroutine InitializeParameters()


	!	read parameters

	open(1, file='Input\Parameters.txt', status = 'old')

	read(1, *) Prod
	read(1, *) irate
	read(1, *) alpha
	read(1, *) bta
	read(1, *) gama
	read(1, *) lambda
	read(1, *) B_leisure
	read(1, *) UI_scale
	read(1, *) UI_pow
	read(1, *) kappa_scale
	read(1, *) kappa_pow
	read(1, *) match_scale
	read(1, *) match_pow
	read(1, *) aveK
	read(1, *) stdK
	read(1, *) kmin
	read(1, *) kmax
	read(1, *) aveE
	read(1, *) emin
	read(1, *) emax
	read(1, *) rhop
	read(1, *) sigep
	read(1, *) amin
	read(1, *) amax
    read(1, *) sfac_wage
    read(1, *) sfac_th
    read(1, *) sfac_K
    read(1, *) sfac_E


	!	monthly interest rate and depreciation rate
	
	irate = (1.0D0+irate)**(1.0D0/12.0D0) - 1.0D0
	bta_J = 1.0D0/(1.0D0+irate)
	
	
	!	std and mean of unconditional distribution of log_x

	UI = UI_scale*Prod**UI_pow
	kappa = kappa_scale*Prod**kappa_pow
	

	!	write parameters on log file

	open(0, file='Output\Logfile.txt', status='unknown')

	write(0, '(A)') "Model Parameters:"
	write(0, '(A,F12.8)') "Prod         = ", Prod
	write(0, '(A,F12.8)') "irate        = ", irate
	write(0, '(A,F12.8)') "alpha        = ", alpha
	write(0, '(A,F12.8)') "bta          = ", bta
	write(0, '(A,F12.8)') "gama         = ", gama
	write(0, '(A,F12.8)') "lambda       = ", lambda
	write(0, '(A,F12.8)') "B_leisure    = ", B_leisure
	write(0, '(A,F12.8)') "UI_scale     = ", UI_scale
	write(0, '(A,F12.8)') "UI_pow       = ", UI_pow
	write(0, '(A,F12.8)') "UI           = ", UI
	write(0, '(A,F12.8)') "kappa_scale  = ", kappa_scale
	write(0, '(A,F12.8)') "kappa_pow    = ", kappa_pow
	write(0, '(A,F12.8)') "kappa        = ", kappa
	write(0, '(A,F12.8)') "match_scale  = ", match_scale
	write(0, '(A,F12.8)') "match_pow    = ", match_pow
	write(0, '(A,F12.8)') "aveK         = ", aveK
	write(0, '(A,F12.8)') "stdK         = ", stdK
	write(0, '(A,F12.8)') "aveE         = ", aveE
	write(0, '(A,F12.8)') "rhop         = ", rhop
	write(0, '(A,F12.8)') "sigep        = ", sigep
	write(0, '(A,F12.8)') "sfac_wage    = ", sfac_wage
	write(0, '(A,F12.8)') "sfac_th      = ", sfac_th
	write(0, '(A,F12.8)') "sfac_K       = ", sfac_K
	write(0, '(A,F12.8)') "sfac_E       = ", sfac_E
	write(0, '(/)') 


end subroutine




subroutine InitializePgrid()


	!	construct equally spaced pgrid

	sigp = sigep/dsqrt(1.0D0-rhop**2.0D0)
	pmax = 2.0D0*sigp
	pmin = -pmax
	step = (pmax-pmin)/(np-1)

	pgrid(1) = pmin
	do ip = 2, np
		pgrid(ip) = pgrid(ip-1) + step
	end do
	pgrid = dexp(pgrid)


	! save grid

	open(1, file='Output\pgrid.txt', status='unknown')
	write(1, '(f12.6)') pgrid
	close(1)


end subroutine



subroutine InitializeKgrid()


	!	construct equally spaced kgrid

	kmin = aveK*(1.0D0-kmin)
	kmax = aveK*(1.0D0+kmax)
	step = (kmax-kmin)/(nk-1)

	kgrid(1) = kmin
	do ik = 2, nk
		kgrid(ik) = kgrid(ik-1) + step
	end do


	! save grid

	open(1, file='Output\Kgrid.txt', status='unknown')
	write(1, '(f12.6)') kgrid
	close(1)


end subroutine



subroutine InitializeEgrid()


	!	construct equally spaced egrid

	emin = aveE*(1.0D0-emin)
	emax = aveE*(1.0D0+emax)
	step = (emax-emin)/(ne-1)

	egrid(1) = emin
	do ie = 2, ne
		egrid(ie) = egrid(ie-1) + step
	end do


	! save grid

	open(1, file='Output\Egrid.txt', status='unknown')
	write(1, '(f12.6)') egrid
	close(1)


end subroutine



subroutine InitializeAgrid()

	implicit none

	
	!	construct unequally spaced asset grids

	step = (amax-amin)/(na-1)

	agrid(1) = amin
	do ia = 2, na
		agrid(ia) = agrid(ia-1) + step
	end do


	! save grid

	open(1, file='Output\agrid.txt', status='unknown')
	write(1, '(f12.6)') agrid
	close(1)
	
	
end subroutine



subroutine InitializeCoef()


	!	read coefficients of theta equation and law of motion for aggregate capital

	open(1, file='Input\Coef_In.txt', status = 'old')
	read(1, *) Coef_th
	read(1, *) Coef_K
	read(1, *) Coef_E
	close(1)


end subroutine



subroutine ComputeThetaKpEp()


	!	compute theta and kp using coefficients


	do ik = 1, nk
	do ie = 1, ne
	do ip = 1, np
	
		theta(ik,ie,ip) = dexp(coef_th(1) + coef_th(2)*dlog(kgrid(ik)) + coef_th(3)*dlog(egrid(ie)) + coef_th(4)*dlog(pgrid(ip)))
		kp(ik,ie,ip)    = dexp(coef_K(1) + coef_K(2)*dlog(kgrid(ik)) + coef_K(3)*dlog(egrid(ie)) + coef_K(4)*dlog(pgrid(ip)))
		ep(ik,ie,ip)    = dexp(coef_E(1) + coef_E(2)*dlog(kgrid(ik)) + coef_E(3)*dlog(egrid(ie)) + coef_E(4)*dlog(pgrid(ip)))
		pth(ik,ie,ip)   = match_scale*theta(ik,ie,ip)**match_pow
	
	end do
	end do
	end do


end subroutine



subroutine InitializeWageValueDecision()


	real(8):: betapth(nk,ne,np), Util_W, Util_U


	!	initialize value functions and Nash-bargaining wages
	!	compute Nash-bargaining wage for a assuming that a is fixed.

	betapth = 1.0D0/(1.0D0-bta*(1.0D0-lambda)+bta*pth)

	do ip = 1, np
	do ie = 1, ne
	do ik = 1, nk
	do ia = 1, na

		!	initialize asset decision rules

		AE(ia,ik,ie,ip) = agrid(ia)
		AU(ia,ik,ie,ip) = agrid(ia)


		!	solve Nash-bargaining wages given AE and AU

		Wage(ia,ik,ie,ip) = Prod*pgrid(ip)


		!	initialize value functions given AE, AU and Wage

		income = (1.0D0+irate)*agrid(ia) + Wage(ia,ik,ie,ip)
		util_leisure = 0.0D0
		Util_W = Utility(AE(ia,ik,ie,ip))

		income = (1.0D0+irate)*agrid(ia) + UI
		util_leisure = B_leisure
		Util_U = Utility(AU(ia,ik,ie,ip))

		W(ia,ik,ie,ip) = (Util_W - bta*lambda*betapth(ik,ie,ip)*(Util_W-Util_U))/(1.0D0-bta)
		U(ia,ik,ie,ip) = (Util_U + bta*pth(ik,ie,ip)*betapth(ik,ie,ip)*(Util_W-Util_U))/(1.0D0-bta)
		J(ia,ik,ie,ip) = (Prod*pgrid(ip)-Wage(ia,ik,ie,ip))/(1.0D0-bta_J*(1.0D0-lambda))

	end do
	end do
	end do
	end do


end subroutine



subroutine InitializeAggregateTransition()


	!	transition probability for aggregate productivity

	call TauchenTr(0.0D0, sigp, rhop, dlog(pgrid), trP)
	open(1, file='Output\trP.txt', status='unknown')
	write(1, '(<np>f10.6)') transpose(trP)


	!	cumulative transition probability matrix of aggregate productivity for random drawing

	CtrP(:,1) = trP(:,1)
	do ip = 2, np
		CtrP(:,ip) = CtrP(:,ip-1) + trP(:,ip)
	end do

	open(1, file='Output\CtrP.txt', status='unknown')
	write(1, '(<np>f10.6)') transpose(CtrP)
	close(1)


end subroutine



subroutine InitializeAggregateShockSeries()


	integer:: pseed, jp
	real(8) ushock
	real, external:: sunif


	! set the seed number for random number generator

	pseed = 10


	! generate Zdata

	iPdata(1) = np/2 + 1
	Pdata(1) = pgrid(iPdata(1))

	do time = 2, Nsim

		ushock = sunif(pseed)

		do jp = 1, np

			if (ushock <= CtrP(iPdata(time-1),jp)) then
			    iPdata(time) = jp
				exit
			end if

		end do

		Pdata(time) = pgrid(iPdata(time))

	end do

	open(1, file='Output\Pdata.txt', status='replace')
	write(1, '(f12.6)') Pdata(Nskip+1:Nsim)


end subroutine



end module
