!	----------------------------------------------------------------------
!	matching model with incomplete market ==> heterogeneity in wealth
!	----------------------------------------------------------------------


program AXKYH_FL

   
	use Globals
	use InitializationModule
	use WageModule


	implicit none
 

	!	set the starting time

	call date_and_time(startdate,starttime)


	!	open a log file

	open(0, file='Output\Logfile.txt', status = 'unknown')


	!	initialization

	call InitializeParameters()


	!	initialize grids

	call InitializeAgrid()
	call InitializeXgrid()
	call InitializeKgrid()
	call InitializeEgrid()
	call InitializePgrid()
	
	
	!	initialize transition matrices for idiosyncratic and aggregate shocks

	call InitializeIdiosyncraticTransition()
	call InitializeAggregateTransition()


	!	initialize aggregate shock series

	call InitializeAggregateShockSeries()


	!	initialize coefficients

	call InitializeCoef()


	!	compute theta and kp

	call ComputeThetaKpEp()


	!	initialize wages and value functions

	call InitializeWageValueDecision()


	!	iterate to find theta equation and law of motion for K

	panel = .false.
	err_th = 1.0D0
	err_K = 1.0D0
	err_E = 1.0D0
	iter_coef = 0

	do while (err_th > tol_th .or. err_K > tol_K .or. err_E > tol_E)


		!	compute theta and kp

		call ComputeThetaKpEp()


		!	find equilibrium wage given theta and kp

		call date_and_time(wage_date,wage_time)
		write(0, '(12A)') "EquilibriumWage started at  ", wage_date(1:4),"-",wage_date(5:6),"-",wage_date(7:8)," ", wage_time(1:2),":",wage_time(3:4),":",wage_time(5:6)

		call EquilibriumWage()
		call SaveValueDecisionWage()

		call date_and_time(wage_date,wage_time)
		write(0, '(12A)') "EquilibriumWage finished at  ", wage_date(1:4),"-",wage_date(5:6),"-",wage_date(7:8)," ", wage_time(1:2),":",wage_time(3:4),":",wage_time(5:6)


		!	simulate the model

		call date_and_time(sim_date,sim_time)
		write(0, '(12A)') "Simulation started at  ", sim_date(1:4),"-",sim_date(5:6),"-",sim_date(7:8)," ", sim_time(1:2),":",sim_time(3:4),":",sim_time(5:6)

		call Simulation()
		call SaveTimeSeries()

		call date_and_time(sim_date,sim_time)
		write(0, '(12A)') "Simulation finished at  ", sim_date(1:4),"-",sim_date(5:6),"-",sim_date(7:8)," ", sim_time(1:2),":",sim_time(3:4),":",sim_time(5:6)


		!	regress the simulated data 

		call RegressCoef()


		!	compute improvements of coefficients and update coefficients 

		err_th = maxval(dabs(coef_th_new - coef_th))
		err_K = maxval(dabs(coef_K_new - coef_K))
		err_E = maxval(dabs(coef_E_new - coef_E))

		coef_th = sfac_th*coef_th_new + (1.0D0-sfac_th)*coef_th
		coef_K = sfac_K*coef_K_new + (1.0D0-sfac_K)*coef_K
		coef_E = sfac_E*coef_E_new + (1.0D0-sfac_E)*coef_E

		iter_coef = iter_coef + 1


		!	show progress on screen and print on log file

		write(0, '(/A,I6)') "Main iteration ", iter_coef
		write(0, '(A,D15.6)') "err_th = ", err_th
		write(0, '(A,D15.6)') "err_K  = ", err_K
		write(0, '(A,D15.6)') "err_E  = ", err_E
		write(0,'(//)')


!		if (iter_coef >= int(7.0D0/sfac_th)) then
		if (iter_coef >= 20) then
			exit
		end if
		
	end do


	!	show converged coefficients

	write(0, '(A,I4)') "Main iteration has finished at iteration ", iter_coef
	write(0, '(A)') " "
	write(0, '(A,<ncoef>D20.10)') "coef_th: ", coef_th
	write(0, '(A,<ncoef>D20.10)') "coef_K:  ", coef_K
	write(0, '(A,<ncoef>D20.10)') "coef_E:  ", coef_E
	write(0, '(A)') " "


	!	run the final simulation and generate panel data

	panel = .false.
	call Simulation()
	call SaveTimeSeries()


	!	set the finishing time

	call date_and_time(finishdate,finishtime)


	!	print the starting and finishing times

	write(*, '(/)') 
	write(*, '(12A)') "The program started at  ", startdate(1:4),"-",startdate(5:6),"-",startdate(7:8)," ", starttime(1:2),":",starttime(3:4),":",starttime(5:6)
	write(*, '(12A)') "The program finished at ", finishdate(1:4),"-",finishdate(5:6),"-",finishdate(7:8)," ", finishtime(1:2),":",finishtime(3:4),":",finishtime(5:6)

	write(0, '(/)') 
	write(0, '(12A)') "The program started at  ", startdate(1:4),"-",startdate(5:6),"-",startdate(7:8)," ", starttime(1:2),":",starttime(3:4),":",starttime(5:6)
	write(0, '(12A)') "The program finished at ", finishdate(1:4),"-",finishdate(5:6),"-",finishdate(7:8)," ", finishtime(1:2),":",finishtime(3:4),":",finishtime(5:6)


end program AXKYH_FL