!	----------------------------------------------------------------------
!	File name: Globals.f90
!
!	Defines global variables.
!	----------------------------------------------------------------------


module Globals

	implicit none

	save


	!	convergence check

	real(8), parameter:: tol_value = 1.0D-4, tol_wage = 1.0D-4, tol_measure = 1.0D-8
	real(8):: err_value, err_W, err_U, err_wage, err_measure, sfac_wage


	!	model parameters to be read

	real(8):: Prod, irate, alpha, bta, gama, lambda, UI, B_leisure, kappa, match_scale, match_pow, log_mean_x, sigex, rhox, bta_J


	!	grids for individual asset holdings

	integer, parameter:: na = 41, dna = 2001
	real(8):: agrid(na), step_a, amin, amax, dagrid(dna)
	real(8):: ave_asset, std_asset
	

	!	grids for idiosyncratic productivity

	integer, parameter:: nx = 15, dnx = 25
!	integer, parameter:: nx = 31, dnx = 61
	integer, parameter:: prod_new_match = nx/2+1
	real(8), parameter:: xbnd = 2.0D0
	real(8):: xgrid(nx), lxgrid(nx), mean_log_x, sigx, lxmax, lxmin, lxstep


	!	transition probability matrix

	real(8):: trX(nx,nx), prX(nx), CtrX(nx,nx), CprX(nx)


	!	auxiliary variables
	
	real(8):: income, util_leisure


	!	theta, matching probabilities

	real(8):: theta, pth, qth, qth_fe, theta_fe


	!	wages

	real(8):: Wage(na,nx)


	!	value functions

	real(8):: W(na,nx), U(na), J(na,nx), DJ(dna,nx), VM, EmaxJG(dna), EmaxJF(dna,nx)


	!	measures

	real(8):: mu(dna,nx), phi(dna), newmu(dna,nx), newphi(dna)
	real(8):: muhat(dna,nx), phihat(dna), mutilde(dna,nx), phitilde(dna)


	!	distributions of asset and earnings
	
	real(8):: earnings(dnx), AsstEarnDensity(dna,0:dnx), AsstDensity(dna), EarnDensity(0:dnx), AsstEarnDist(dna,0:dnx), AsstDist(dna), EarnDist(0:dnx)


	!	worker's asset and consumption decision rules

	real(8):: AE(na,nx), AU(na), CE(na,nx), CU(na)


	!	reservation productivity for endogenous match separations

	integer:: xr(na), cutoff_a(nx), cutoff_x(dna)


	!	firm's vacancy decision rule

	real(8):: vac


	!	indexing

	integer:: ia, ix, jx, iter_value, iter_wage, iter_measure


	!	timer

	character(8):: startdate, finishdate
	character(10):: starttime, finishtime


	!	aggregate variables
	
	real(8):: E_x(nx), U_x(nx), E_a(dna), U_a(dna)
	real(8):: empl, unempl, EtoU, UtoE, hEtoU, hUtoE, empl_dur, unempl_dur,	 ave_asset_E, ave_asset_U, std_asset_E, std_asset_U, ave_wage, std_wage, dwage(dna,nx), ave_lwage, std_lwage
	
end module
