subroutine SimulatePanel()

	use Globals
	use LinInterpModule


	implicit none


	integer, parameter:: Nindiv = 10000, Nindiv_panel = 1000, Nsim = 1000, Nskip = 400
	integer:: indiv, time, time2
	integer:: xseed
	real(8):: cscoef_wage(na,4,nx), cscoef_W(na,4,nx), cscoef_U(na,4,nx)
	type Individual
		real(8):: A, AP, W, C, lambdashock, pthshock
		integer:: E, EP, X, XP
	end type Individual
	type(Individual), dimension(Nindiv):: Worker
	real(8), allocatable:: lambdavec(:), pthvec(:)
	integer, allocatable:: lambda_iperm(:), pth_iperm(:)
	real(8):: cutoff_lambda, cutoff_pth, xshock
	integer:: ind_E, ind_U, Nind_E, Nind_U, IER
	real, external:: sunif, snorm
	real(8), external:: csval


	!	set the seed number for random number generator

	xseed = 2


	!	intialize asset distribution of workers

	do indiv = 1, Nindiv
	
		Worker(indiv)%A = min(max(ave_asset_E + std_asset_E*snorm(xseed), agrid(1)), agrid(na))

	end do


    !   initialize employment status
    
    do indiv = 1, Nindiv
    
        xshock = sunif(xseed)
        if (xshock <= empl) then
            Worker(indiv)%E = 1
        else
            Worker(indiv)%E = 0
        end if
        
    end do
    

    !   initialize productivity
    
	do indiv = 1, Nindiv
	
	    if (Worker(indiv)%E == 1) then
	    
	        xshock = sunif(xseed)
    		do ix = 1, nx
                if (xshock <= CprX(ix)) then
                    Worker(indiv)%X = ix
                    exit
                end if
            end do

        end if

	end do


	!	interpolate wages

	do ix = 1, nx
	
		call csint(na, agrid, Wage(:,ix), cscoef_wage(:,:,ix))
		call csint(na, agrid, W(:,ix), cscoef_W(:,:,ix))
	
	end do

	call csint(na, agrid, U, cscoef_U)


	!	start simulation 

	do time = 1, Nsim


		!	allocate memory for random numbers
		
		Nind_E = sum(Worker%E,mask=Worker%E==1)
		Nind_U = sum(1-Worker%E,mask=Worker%E==0)

		allocate(lambdavec(Nind_E), pthvec(Nind_U), lambda_iperm(Nind_E), pth_iperm(Nind_U))


		!	individual's asset holdings in the next period, assign random number for next period match status

		ind_E = 0
		ind_U = 0

		do indiv = 1, Nindiv

			if (Worker(indiv)%E == 1) then

				Worker(indiv)%W  = csval(na, Worker(indiv)%A, agrid, cscoef_wage(:,:,Worker(indiv)%X))
				Worker(indiv)%AP = min(max(lininterp1(Worker(indiv)%A, agrid, AE(:,Worker(indiv)%X)), agrid(1)), agrid(na))
       			Worker(indiv)%C  = (1.0D0+irate)*Worker(indiv)%A + Worker(indiv)%W - Worker(indiv)%AP

				ind_E = ind_E + 1
				lambdavec(ind_E) = sunif(xseed)
				Worker(indiv)%lambdashock = lambdavec(ind_E)

			else

				Worker(indiv)%AP = min(max(lininterp1(Worker(indiv)%A, agrid, AU), agrid(1)), agrid(na))
       			Worker(indiv)%C  = (1.0D0+irate)*Worker(indiv)%A + UI - Worker(indiv)%AP

				ind_U = ind_U + 1
				pthvec(ind_U) = sunif(xseed)
				Worker(indiv)%pthshock = pthvec(ind_U)

			end if

		end do


		!	sort lambdavec and pthvec

		call sort(lambdavec, Nind_E, lambda_iperm, 2, IER)
		cutoff_lambda = lambdavec(int(lambda*Nind_E))
		call sort(pthvec, Nind_U, pth_iperm, 2, IER)
		cutoff_pth = pthvec(int(pth*Nind_U))

		deallocate(lambdavec, pthvec, lambda_iperm, pth_iperm)


		!	assign match status in the next period

		do indiv = 1, Nindiv
	
			if (Worker(indiv)%E == 1) then
				
				if (Worker(indiv)%lambdashock <= cutoff_lambda) then
					Worker(indiv)%XP = -9
				else
        	        xshock = sunif(xseed)
    	        	do ix = 1, nx
                        if (xshock <= CtrX(Worker(indiv)%X,ix)) then
                            Worker(indiv)%XP = ix
                            exit
                        end if
                    end do
				end if

			else

				if (Worker(indiv)%pthshock <= cutoff_pth) then
				    Worker(indiv)%XP = prod_new_match
				else
					Worker(indiv)%XP = -9
				end if

			end if

		end do


        !   worker's employment decision in the next period

        do indiv = 1, Nindiv
        
			if (Worker(indiv)%XP > 0) then

				if (csval(na, Worker(indiv)%AP, agrid, cscoef_W(:,:,Worker(indiv)%XP)) >= csval(na, Worker(indiv)%AP, agrid, cscoef_U)) then
					Worker(indiv)%EP = 1
                else
					Worker(indiv)%EP = 0
                end if
             
            else
            
                Worker(indiv)%EP = 0   
            
            end if

        end do


		!	show aggregate variables on screen

		if (mod(time,10) == 0) then
		
			write(*, '(A,I7)')    "time      = ", time
			write(*, '(A,F12.5)') "Kdata     = ", sum(Worker%A)/Nindiv
			write(*, '(A,F12.5)') "Edata     = ", dble(Nind_E)/Nindiv
			write(*, '(A,F12.5)') "EUdata    = ", dble(sum(Worker%E,mask=Worker%EP==0))/Nindiv
			write(*, '(A,F12.5)') "UEdata    = ", dble(sum(Worker%EP,mask=Worker%E==0))/Nindiv
			write(*, '(A)') " "
		
		end if


        !   save aggregate variables for time > Nskip
        
		if (time > Nskip) then

			time2 = time - Nskip
			
			!	save panel data

			open(100, file='Output\panel.txt', status='unknown')

			do indiv = 1, Nindiv_panel
				
				write(100, '(I3, I6, F8.3, I3, 2F7.3, I3)') time2, indiv, Worker(indiv)%A, Worker(indiv)%X, Worker(indiv)%W, Worker(indiv)%C, Worker(indiv)%E
	
			end do

    	end if


		!	update employment status in the previous period

		Worker%E = Worker%EP
		Worker%A = Worker%AP
		Worker%X = Worker%XP
		Worker%W = 0.0D0
		Worker%C = 0.0D0


	end do

	close(100)


end subroutine
