!	----------------------------------------------------------------------
!	Module name : ValueModule.f90
!	----------------------------------------------------------------------	*/


module ValueModule

	use Globals
	use optimizationMod
    

	implicit none

	real(8):: cscoef(na,4), cscoef_W(na,4), cscoef_U(na,4)
	real(8), external:: csval
	

contains


subroutine SolveValueFunctions()


	real(8):: NewW(na,nx), NewU(na), maxWU(nx), EmaxWUF(na,nx), EmaxWUG(na)
	real(8):: EV_W(na,nx), EV_U(na), ae_opt, au_opt, lbd, ubd


	!	start iteration

	err_value = 1.0D0
	iter_value = 0

	do while (err_value > tol_value)


		!	expected values in the next period according to the current employment status

		do ia = 1, na

			!	Emax[W(a,x'), U(a)|F] when survives lambda shock

			do jx = 1, nx
				maxWU(jx) = max(W(ia,jx), U(ia))
			end do

			do ix = 1, nx

				EmaxWUF(ia,ix) = dot_product(maxWU, trX(ix,:))
			
			end do


			!	Emax[W(a,x'), U(a)|G] when survives lambda shock

			EmaxWUG(ia) = dot_product(maxWU, prX)
			
		end do


		!	expected values

		do ix = 1, nx
			EV_W(:,ix) = (1.0D0-lambda)*EmaxWUF(:,ix) + lambda*U
		end do

		EV_U = pth*EmaxWUG + (1.0D0-pth)*U


		!	employed worker's utility maximization

		do ix = 1, nx

			call csint(na, agrid, EV_W(:,ix), cscoef_W)
			cscoef = cscoef_W

			do ia = 1, na

				income = (1.0D0+irate)*agrid(ia) + Wage(ia,ix)
				lbd = amin
				ubd = income
				util_leisure = 0.0D0
				call fmin(NegValueF, lbd, ubd, ae_opt)
				AE(ia,ix) = ae_opt
				CE(ia,ix) = income - AE(ia,ix)
				NewW(ia,ix) = ValueF(AE(ia,ix))
		
			end do
	
		end do


		!	unemployed worker's utility maximization

		call csint(na, agrid, EV_U, cscoef_U)
		cscoef = cscoef_U

		do ia = 1, na

			income = (1.0D0+irate)*agrid(ia) + UI
			lbd = amin
			ubd = income
			util_leisure = B_leisure
			call fmin(NegValueF, lbd, ubd, au_opt)
			AU(ia) = au_opt
			CU(ia) = income - AU(ia)
			NewU(ia) = ValueF(AU(ia))

		end do


		!	calculate improvements of value functions

		err_W = maxval(dabs(NewW - W))
		err_U = maxval(dabs(NewU - U))
		err_value = max(err_W, err_U)


		!	update value functions

		W = NewW
		U = NewU
		
		iter_value = iter_value + 1


		!	exit 
		
		if (err_U <= 1.0D-6) then
			exit
		end if
		
		
		!	show progress

		if (mod(iter_value,500) == 0) then
			write(*, '(A,I 6)') "SolveValueFunction iteration ", iter_value
			write(*, '(2(A,F10.6))') "err_W = ", err_W, "   err_U = ", err_U
			print*, " "
		end if

	end do


	print*, "Worker's value functions have converged at iteration " ,iter_value
	print*, " "


end subroutine



real(8) function Utility(ap)


	real(8), parameter:: tiny = 1.0D-20
	real(8):: cons, uh, ap

	cons = max(income - ap, tiny)

    if (gama == 1.0D0) then
    	Utility = dlog(cons) + util_leisure
    else
    	Utility = (cons**(1.0D0-gama) - 1.0D0)/(1.0D0-gama) + util_leisure
    end if
    

end function



real(8) function ValueF(ap)


	real(8):: ap

	ValueF = Utility(ap) + bta*csval(na, ap, agrid, cscoef)

end function



real(8) function NegValueF(ap)


	real(8):: ap

	NegValueF = -ValueF(ap)

end function


end module
