!	----------------------------------------------------------------------
!	File name : SteadyStateFixedDiscrete.f90
!
!	Solves heterogeneous agent model in which worker's productivity (x,z)
!	is changing over time.
!	----------------------------------------------------------------------


program A_SS

    
	use Globals
	use InitializationModule
	use WageModule
	use MeasureModule

	implicit none
 
 	
	!	set the starting time

	call date_and_time(startdate,starttime)


	!	initialization: parameters, asset grids

	call InitializeParameters()
	call InitializeAgrid()


	!	initialize value functions and Nash-bargaining wages

	call InitializeWageValueDecision()
	
	
	!	find equilibrium Nash-bargaining wages

	call EquilibriumWage()
	

	!	compute time-invariant measures

	call InvariantMeasures()


	!	compute newtheta using free entry condition

	do ia = 1, dna
		DJ(ia) = csval(na, dagrid(ia), agrid, cscoef_J)
	end do
	VM = dot_product(DJ, phi/sum(phi))
	qth = kappa/(bta_J*VM)
	theta = (match_scale/qth)**(1.0D0/(1.0D0-match_pow))

	write(*, '(3(A,F10.6))') "kappa=", kappa, "   qth= ", qth, "   theta= ", theta


	!	compute aggregate variables

	call AggregateVariables()


	!	save all results

	call SaveResults()


	!	set the finishing time

	call date_and_time(finishdate,finishtime)


	!	print the starting and finishing times

	write(*, '(/)') 
	write(*, '(12A)') "The program started at  ", startdate(1:4),"-",startdate(5:6),"-",startdate(7:8)," ", starttime(1:2),":",starttime(3:4),":",starttime(5:6)
	write(*, '(12A)') "The program finished at ", finishdate(1:4),"-",finishdate(5:6),"-",finishdate(7:8)," ", finishtime(1:2),":",finishtime(3:4),":",finishtime(5:6)


end program A_SS