!	----------------------------------------------------------------------
!	Module name : ValueModule.f90
!	----------------------------------------------------------------------	*/


module ValueModule

	use Globals
	use optimizationMod
	use LinInterpModule
    
    
	implicit none

	real(8):: cscoef(na,4), cscoef_W(na,4), cscoef_U(na,4)
	real(8), external:: csval
	

contains


subroutine SolveValueFunctions()


	real(8):: NewW(na), NewU(na)
	real(8):: EV_W(na), EV_U(na), xmin, lbd, ubd


	!	start iteration

	err_value = 1.0D0
	iter_value = 0

	do while (err_value > tol_value)


		!	expected values in the next period according to the current employment status
	
		EV_W = (1.0D0-lambda)*W + lambda*U
		EV_U = pth*W + (1.0D0-pth)*U


		!	employed worker's utility maximization

		call csint(na, agrid, EV_W, cscoef_W)
		cscoef = cscoef_W

		do ia = 1, na

			income = (1.0D0+irate)*agrid(ia) + Wage(ia)
			util_leisure = 0.0D0
			lbd = amin
			ubd = income
			call fmin(NegValueF, lbd, ubd, xmin)
			AE(ia) = xmin
			CE(ia) = income - AE(ia)
			NewW(ia) = ValueF(AE(ia))
		
		end do


		!	unemployed worker's utility maximization

		call csint(na, agrid, EV_U, cscoef_U)
		cscoef = cscoef_U

		do ia = 1, na

			income = (1.0D0+irate)*agrid(ia) + UI
			util_leisure = B_leisure
			lbd = amin
			ubd = income
			call fmin(NegValueF, lbd, ubd, xmin)
			AU(ia) = xmin
			CU(ia) = income - AU(ia)
			NewU(ia) = ValueF(AU(ia))

		end do


		!	calculate improvements of value functions

		err_W = maxval(dabs(NewW - W))
		err_U = maxval(dabs(NewU - U))
		err_value = max(err_W, err_U)


		!	update value functions

		W = NewW
		U = NewU
		
		iter_value = iter_value + 1


		!	show progress

		if (mod(iter_value,1000) == 0) then
			write(*, '(A,I6)') "SolveValueFunction iteration ", iter_value
			write(*, '(2(A,F10.6))') "err_W = ", err_W, "   err_U = ", err_U
			print*, " "
		end if

	end do


	print*, "Worker's value functions have converged at iteration " ,iter_value
	print*, " "

end subroutine



real(8) function Utility(ap)


	real(8), parameter:: tiny = 1.0D-20
	real(8):: cons, uh, ap

	cons = max(income - ap, tiny)

	Utility = dlog(cons) + util_leisure
!	Utility = (cons**(1.0D0-gama) - 1.0D0)/(1.0D0-gama)

end function



real(8) function ValueF(ap)


	real(8):: ap

	ValueF = Utility(ap) + bta*csval(na, ap, agrid, cscoef)

end function



real(8) function NegValueF(ap)


	real(8):: ap

	NegValueF = -ValueF(ap)

end function


end module
