module WageModule


	use Globals
	use ValueModule
	use OptimizationMod


	implicit none

	real(8):: cscoef_J(na,4)


contains


subroutine EquilibriumWage()


	real(8):: NewWage(na)


	!	start iterations

	err_wage = 1.0D0
	iter_wage = 0

	do while (err_wage > tol_wage)


		!	compute value functions given wages

		call SolveValueFunctions()


		!	update wages using Nash bargaining condition: J = (1.0D0-alpha)/alpha*(W - U)*ce

		J = (1.0D0-alpha)/alpha*(W - U)*CE
		call csint(na, agrid, J, cscoef_J)
		
		do ia = 1, na
			NewWage(ia) = Prod - J(ia) + bta_J*(1.0D0-lambda)*csval(na, AE(ia), agrid, cscoef_J)
		end do


		!	compute improvement of wages

		err_wage = maxval(dabs(NewWage - Wage))

		
		!	update wages

		Wage = sfac_wage*NewWage + (1.0D0-sfac_wage)*Wage

		iter_wage = iter_wage + 1


		!	show progress

		print*, "EquilibriumWage iteration ", iter_wage
		print*, "err_wage = ", err_wage
		print*, " "

	end do


	!	print the final iteration number

	print*, "Wages have converged at iteration " ,iter_wage
	print*, " "


end subroutine



end module
