!	----------------------------------------------------------------------
!	file name: opimization.f90
!
!	translation of Matlab function fmin('F', x1, x2)
!	algorithm is very similar to "fminbr" from NETLIB
!	----------------------------------------------------------------------	*/


module optimizationMod

contains



real(8) function bisect(func,x1,x2,xacc)

	implicit none

	integer, parameter:: JMAX = 100
	real(8) rtbis, x1, x2, xacc, func, dx, f, fmid, xmid
	external func
	integer j


	fmid=func(x2)
	f=func(x1)
	

	if(f*fmid >= 0.0D0) then
		print*, "root must be bracketed in rtbis"
		stop
	end if


	if(f < 0.0D0)then
		rtbis=x1
		dx=x2-x1
	else
		rtbis=x2
		dx=x1-x2
	endif


	do j = 1, JMAX

		dx=dx*0.5D0
		xmid=rtbis+dx
		fmid=func(xmid)
		if(fmid <= 0.0D0) rtbis = xmid
!		if(dabs(dx) < xacc .or. fmid == 0.0D0) then
		if(dabs(dx) < xacc .or. dabs(fmid) < xacc) then
			bisect = rtbis
			return
		end if

	end do
    
	print*, "too many bisections in rtbis"


end function





subroutine fmin(fcn, ax, bx, xmin)

	implicit none

	real(8) fcn, ax, bx, xmin
	external fcn

	real(8), parameter:: tol = 1.0D-8, eps = 2.220446049250313D-016
	integer, parameter:: maxiter = 500
	integer num
	real(8) seps, c, a,	b, v, w, xf, d,	e, x, fx, fv, fw, xm, tol1, tol2, p, q, r, fu
	logical	gs


	if (ax == bx) then
		bx = ax + 1.0D-20
	end if


	!	initialization

	num = 1
	seps = dsqrt(eps)
	c = 0.5D0*(3.0D0 - dsqrt(5.0D0))
	a = ax
	b = bx
	v = a + c*(b-a)
	w = v
	xf = v
	d = 0.0D0
	e = 0.0D0
	x = xf
	fx = fcn(x)
	fv = fx
	fw = fx
	xm = 0.5D0*(a + b)
	tol1 = seps*dabs(xf) + tol/3.0D0
	tol2 = 2.0D0*tol1
	gs = .false.


	!	main loop

	do while ( dabs(xf - xm) > (tol2 - 0.5D0*(b-a)) )

		num = num + 1
		gs = .true.

		!	is a parabolic fit possible

		if (dabs(e) > tol1) then

			!	yes, so fit parabola

			gs = .false.
			r = (xf - w)*(fx - fv)
			q = (xf - v)*(fx - fw)
			p = (xf - v)*q - (xf - w)*r
			q = 2.0D0*(q - r)
			if (q > 0) then
				p = -p
			end if
			q = dabs(q)
			r = e
			e = d

			!	is parabola acceptable

			if ((dabs(p) < dabs(0.5D0*q*r)) .and. (p > q*(a-xf)) .and.		&
				(p < q*(b-xf))) then
				
				!	yes, parabolic interpolation step

				d = p/q
				x = xf + d

				!	f must not be evaluated too close to ax or bx

				if ( ((x-a) < tol2) .or. ((b-x) < tol2) ) then
					if (xm >= xf) then 
						d = tol1
					else 
						d = -tol1
					end if
				end if

			else

				!	no, not acceptable, must do a golden section step
			
				gs = .true.

			end if

		end if

		if (gs) then

			!	a golden section step is required

			if (xf >= xm) then
				e = a - xf
			else
				e = b - xf
			end if
		
			d = c*e

		end if

		!	the function must not be evaluated to close to xf
		
		if (d >= 0) then
			x = xf + max(dabs(d), tol1)
		else 
			x = xf - max(dabs(d), tol1)
		end if

		fu = fcn(x)


		!	update a, b, v, w, x, xm, tol1, tol2

		if (fu <= fx) then

			if (x >= xf) then
				a = xf
			else
				b = xf
			end if
		
			v = w
			fv = fw
			w = xf
			fw = fx
			xf = x
			fx = fu

		else

			if (x < xf) then
				a = x
			else
				b = x
			end if

			if ( fu <= fw .or. w == xf ) then
				v = w
				fv = fw
				w = x
				fw = fu
			else if ( fu <= fw .or. v == xf .or. v == w ) then
				v = x;
				fv = fu;
			end if

		end if

		xm = 0.5D0*(a + b)
		tol1 = seps*dabs(xf) + tol/3.0D0
		tol2 = 2.0D0*tol1

		if (num > maxiter) then
			print*, "Warning: Maximum number of function evaluation has been exceeded"
			stop
		end if

	end do
	
	xmin = x

end subroutine
	
end module
